@@ -13,9 +13,11 @@ has dir => (
     isa => 'Path::Class::Dir',
     required => 1,
     coerce => 1,
-    default => sub { File::HomeDir->my_home . ".backimap" },
+    builder => '_build_dir',
 );
 
+sub _build_dir { return File::HomeDir->my_home . ".backimap" }
+
 
 has init => (
     is => 'ro',
@@ -31,6 +33,13 @@ has clean => (
 );
 
 
+has resume => (
+    is => 'ro',
+    isa => 'Bool',
+    default => 0,
+);
+
+
 has author => (
     is => 'ro',
     isa => 'Str',
@@ -74,14 +83,25 @@ sub _build_git {
     }
 
     if ( $git->status->is_dirty ) {
-        die "directory $dir is dirty, consider --clean option\n"
-            unless $self->clean;
+        die "directory $dir is dirty, consider --clean or --resume options\n"
+            unless $self->clean or $self->resume;
 
-        _git_reset($git);
+        my @unknown = map { $_->from } $git->status->get("unknown");
 
-        if ( $git->status->is_dirty ) {
-            my @unknown = map { $_->from } $git->status->get("unknown");
-            die "directory $dir still has unknown files: @unknown\n";
+        # --resume takes precedence over --clean
+        if ( $self->resume ) {
+            for my $file (@unknown) {
+                $dir->file($file)->remove();
+            }
+
+            eval { $git->commit( { all => 1, message => 'resume previous backup' } ) }
+                if $git->status->is_dirty;
+        }
+        elsif ( $self->clean ) {
+            _git_reset($git);
+
+            die "directory $dir has unknown files: @unknown\n"
+                if @unknown;
         }
     }
 
@@ -89,6 +109,14 @@ sub _build_git {
 }
 
 
+sub BUILD {
+    # This makes sure that git repo is properly initialized
+    # before returning successfully from new() constructor.
+
+    shift->_git();
+}
+
+
 sub find {
     my $self = shift;
 
@@ -122,12 +150,7 @@ sub get {
 sub put {
     my $self = shift;
     my %files = @_;
-
-    # This makes sure that git repo is properly initialized
-    # before any new file is added. Otherwise it would fail
-    # because repo would be dirty.
     my $git = $self->_git;
-
     my $dir = $self->dir;
 
     for my $filename ( keys %files ) {
@@ -148,10 +171,11 @@ sub put {
 
 sub delete {
     my $self = shift;
+    my $git = $self->_git;
 
-    my @files = map { $self->dir->file($_)->stringify() } @_;
+    my @files = map { "$_" } @_;
 
-    $self->_git->rm(@files)
+    $git->rm(@files)
         if @files;
 }
 
@@ -159,20 +183,22 @@ sub delete {
 sub move {
     my $self = shift;
     my ( $from, $to ) = @_;
+    my $git = $self->_git;
 
-    $self->_git->mv( $from, $to );
+    $git->mv( $from, $to );
 }
 
 
 sub commit {
     my $self = shift;
     my $change = shift;
+    my $git = $self->_git;
 
     if (@_) {
-        $self->_git->commit( { message => $change }, @_ );
+        $git->commit( { message => $change }, @_ );
     }
     else {
-        $self->_git->commit( { message => $change, all => 1 } );
+        $git->commit( { message => $change, all => 1 } );
     }
 }
 
@@ -196,7 +222,7 @@ App::backimap::Storage - manages backimap storage
 
 =head1 VERSION
 
-version 0.00_10
+version 0.00_11
 
 =head1 ATTRIBUTES
 
@@ -212,6 +238,12 @@ Tells that storage must be initialized.
 
 Tells that storage must be cleaned if dirty.
 
+=head2 resume
+
+Tells that storage should try to resume from a dirty state:
+preserve previous scheduled files and purge unknown ones
+before performing a commit.
+
 =head2 author
 
 Name of the committing author in local storage.
@@ -247,21 +279,23 @@ Adds files to storage with a text describing the change.
 
 Removes files from storage.
 
-=head2 move
+=head2 move( $from, $to )
 
 Renames or moves files and directories from one place to another in storage.
 
-=head2 commit($change, [$file] ...)
+=head2 commit( $change, [$file] ... )
 
 Commits pending storage actions with a description of change.
 If a list of files is provided, only those will be committed.
 Otherwise all pending actions will be performed.
 
-=head2 reset
+=head2 reset()
 
 Rolls back any storage actions that were performed but not committed.
 Returns storage back to last committed status.
 
+=for Pod::Coverage BUILD
+
 =for Pod::Coverage pack unpack
 
 Required methods in status for MooseX::Storage that don't perform any action
